@@ -16,18 +16,18 @@ our $pars_re = qr/^
 	\s*(?:($complex_regex|$typeregex)\b([+]*)|)\s*	# $1,2: first option then plus
 	(?:$sqbr_re)?\s*	# $3: The initial [option] part
 	(\w+)			# $4: The name
-	\(([^)]*)\)		# $5: The indices
+	\((.*)\)		# $5: The indices
+	\s*\Z			# that's all
 /x;
 my %flag2info = (
   io => [[qw(FlagW)]],
-  nc => [[qw(FlagNCreat)]],
   o => [[qw(FlagOut FlagCreat FlagW)]],
   oca => [[qw(FlagOut FlagCreat FlagW FlagCreateAlways)]],
   t => [[qw(FlagTemp FlagCreat FlagW)]],
   phys => [[qw(FlagPhys)]],
-  real => [[qw(FlagReal)]],
-  complex => [[qw(FlagComplex)]],
-  (map +($_->ppforcetype => [[qw(FlagTyped)], 'Type']), types),
+  real => [[qw(FlagTypeOverride FlagReal)]],
+  complex => [[qw(FlagTypeOverride FlagComplex)]],
+  (map +($_->ppforcetype => [[qw(FlagTypeOverride FlagTyped)], 'Type']), types),
 );
 my %flag2c = qw(
   FlagReal PDL_PARAM_ISREAL
@@ -42,41 +42,48 @@ my %flag2c = qw(
   FlagPhys PDL_PARAM_ISPHYS
   FlagIgnore PDL_PARAM_ISIGNORE
 );
+my $calc_re = qr{
+  (\w+)\s*=\s*      # paren group 1 (dim name) - from perlre/PARNO
+  CALC
+  (                 # paren group 2 (parens)
+    \(
+      (             # paren group 3 (contents of parens)
+      (?:
+       (?> [^()]+ ) # Non-parens without backtracking
+      |
+       (?2)         # Recurse to start of paren group 2
+      )*
+      )
+    \)
+  )
+}xo;
 sub new {
-	my($type,$string,$badflag,$sig) = @_;
-	$badflag ||= 0;
-	my $this = bless {Number => "PDL_UNDEF_NUMBER", BadFlag => $badflag, Sig => $sig},$type;
-	# Parse the parameter string. Note that the regexes for this match were
-	# originally defined here, but were moved to PDL::PP for FullDoc parsing.
-	$string =~ $pars_re
-		 or confess "Invalid pdl def $string (regex $pars_re)\n";
-	my($opt1,$opt_plus,$sqbr_opt,$name,$inds) = map $_ // '', $1,$2,$3,$4,$5;
-	print "PDL: '$opt1$opt_plus', '$sqbr_opt', '$name', '$inds'\n"
-		  if $::PP_VERBOSE;
-	croak "Invalid Pars name: $name"
-	  if $INVALID_PAR{$name};
+  my ($type,$string,$badflag,$sig) = @_;
+  $badflag ||= 0;
+  my $this = bless {Number => "PDL_UNDEF_NUMBER", BadFlag => $badflag, Sig => $sig},$type;
+  $string =~ $pars_re or confess "Invalid pdl def $string (regex $pars_re)\n";
+  my($opt1,$opt_plus,$sqbr_opt,$name,$inds) = map $_ // '', $1,$2,$3,$4,$5;
+  print "PDL: '$opt1$opt_plus', '$sqbr_opt', '$name', '$inds'\n"
+    if $::PP_VERBOSE;
+  croak "Invalid Pars name: $name" if $INVALID_PAR{$name};
 # Set my internal variables
-	$this->{Name} = $name;
-	$this->{Flags} = [(split ',',$sqbr_opt),($opt1?$opt1:())];
-	for(@{$this->{Flags}}) {
-		confess("Invalid flag $_ given for $string\n")
-			unless my ($set, $store) = @{ $flag2info{$_} || [] };
-		$this->{$store} = $_ if $store;
-		$this->{$_} = 1 for @$set;
-	}
-	if ($this->{FlagTyped} && $opt_plus) {
-	  $this->{FlagTplus} = 1;
-	}
-	$this->{Type} &&= PDL::Type->new($this->{Type});
-	if($this->{FlagNCreat}) {
-		delete $this->{FlagCreat};
-		delete $this->{FlagCreateAlways};
-	}
-	$this->{RawInds} = [map{
-		s/\s//g; 		# Remove spaces
-		$_;
-	} split ',', $inds];
-	return $this;
+  $this->{Name} = $name;
+  $this->{Flags} = [(split ',',$sqbr_opt),($opt1?$opt1:())];
+  for(@{$this->{Flags}}) {
+    confess("Invalid flag $_ given for $string\n")
+      unless my ($set, $store) = @{ $flag2info{$_} || [] };
+    $this->{$store} = $_ if $store;
+    $this->{$_} = 1 for @$set;
+  }
+  $this->{FlagTplus} = 1 if $this->{FlagTyped} && $opt_plus;
+  $this->{Type} &&= PDL::Type->new($this->{Type});
+  $this->{Ind2Calc} = \my %ind2calc;
+  $ind2calc{$1} = $3 while $inds =~ s#$calc_re#$1#;
+  $this->{RawInds} = [map{
+    s/\s//g; 		# Remove spaces
+    $_;
+  } split ',', $inds];
+  return $this;
 }
 
 sub cflags {
@@ -84,11 +91,11 @@ sub cflags {
   map $flag2c{$_}, grep $this->{$_}, sort keys %flag2c;
 }
 
-sub name {return (shift)->{Name}}
+sub name {$_[0]{Name}}
 
 sub add_inds {
 	my($this,$dimsobj) = @_;
-	$this->{IndObjs} = [my @objs = map $dimsobj->get_indobj_make($_), @{$this->{RawInds}}];
+	$this->{IndObjs} = [my @objs = map $dimsobj->get_indobj_make($_, $this->{Ind2Calc}{$_}), @{$this->{RawInds}}];
 	my %indcount;
 	$this->{IndCounts} = [ map 0+($indcount{$_->name}++), @objs ];
 	$this->{IndTotCounts} = [ map $indcount{$_->name}, @objs ];
@@ -150,10 +157,6 @@ sub get_nname{ my($this) = @_;
 	"(\$PRIV(pdls)[$this->{Number}])";
 }
 
-sub get_nnflag { my($this) = @_;
-	"(\$PRIV(vtable)->per_pdl_flags[$this->{Number}])";
-}
-
 sub get_substname {
   my($this,$ind) = @_;
   $this->{IndObjs}[$ind]->name.($this->{IndTotCounts}[$ind] > 1 ? $this->{IndCounts}[$ind] : '');
@@ -167,12 +170,12 @@ sub get_incname {
 
 sub get_incregisters {
 	my($this) = @_;
-	if(scalar(@{$this->{IndObjs}}) == 0) {return "";}
-	(join '',map {
+	return '' if scalar(@{$this->{IndObjs}}) == 0;
+	join '', map {
 		my $x = $_;
 		my ($name, $for_local) = map $this->get_incname($x, $_), 0, 1;
-		"register PDL_Indx $for_local = __privtrans->$name; (void)$for_local;\n";
-	} (0..$#{$this->{IndObjs}}) )
+		"register PDL_Indx $for_local = __privtrans->$name; (void)$for_local;";
+	} 0..$#{$this->{IndObjs}};
 }
 
 # Print an access part.
@@ -197,7 +200,7 @@ sub do_access {
 
 sub do_pdlaccess {
 	my($this) = @_;
-	PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls)['.$this->{Number}.']');
+	'$PRIV(pdls)['.$this->{Number}.']';
 }
 
 sub do_pointeraccess {
@@ -205,11 +208,6 @@ sub do_pointeraccess {
 	return $this->{Name}."_datap";
 }
 
-sub do_physpointeraccess {
-	my($this) = @_;
-	return $this->{Name}."_physdatap";
-}
-
 sub do_indterm { my($this,$pdl,$ind,$subst,$context) = @_;
   my $substname = $this->get_substname($ind);
 # See if substitutions
@@ -219,17 +217,15 @@ sub do_indterm { my($this,$pdl,$ind,$subst,$context) = @_;
   confess "Access Index not found: $pdl, $ind, @{[$this->{IndObjs}[$ind]->name]}
 	  On stack:".(join ' ',map {"($_->[0],$_->[1])"} @$context)."\n"
 	  if !defined $index;
-  return "(".($this->get_incname($ind,1))."*".
-	 "PP_INDTERM(".$this->{IndObjs}[$ind]->get_size().", $index))";
+  return "(".($this->get_incname($ind,1))."*($index))";
 }
 
 sub get_xsdatapdecl { 
-    my($this,$ctype,$nulldatacheck) = @_;
+    my($this,$ctype,$nulldatacheck,$ppsym) = @_;
     my $pdl = $this->get_nname;
-    my $flag = $this->get_nnflag;
     my $name = $this->{Name};
     my $macro = "PDL_DECLARE_PARAMETER".($this->{BadFlag} ? "_BADVAL" : "");
-    "$macro($ctype, $flag, $name, $pdl, $nulldatacheck)";
+    "$macro($ctype, $name, $pdl, $nulldatacheck, $ppsym)";
 }
 
 1;
