@@ -23,10 +23,10 @@ Internal module to handle signatures
 sub nospacesplit {grep /\S/, split $_[0],$_[1]}
 
 sub new {
-  my ($type,$str,$bvalflag,$otherpars) = @_;
+  my ($type,$pars,$bvalflag,$otherpars) = @_;
   $bvalflag ||= 0;
   my $this = bless {}, $type;
-  my @objects = map PDL::PP::PdlParObj->new($_,$bvalflag, $this), nospacesplit ';',$str;
+  my @objects = map PDL::PP::PdlParObj->new($_,$bvalflag, $this), nospacesplit ';',$pars;
   $this->{Names} = [ map $_->name, @objects ];
   $this->{Objects} = { map +($_->name => $_), @objects };
   my @objects_sorted = ((grep !$_->{FlagW}, @objects), (grep $_->{FlagW}, @objects));
@@ -34,54 +34,44 @@ sub new {
   $this->{NamesSorted} = [ map $_->name, @objects_sorted ];
   $this->{DimsObj} = my $dimsobj = PDL::PP::PdlDimsObj->new;
   $_->add_inds($dimsobj) for @objects;
-  my (%ind2use, %ind2obj);
-  for my $o (@objects) {
-    for my $io (@{$o->{IndObjs}}) {
-      push @{$ind2use{$io->name}}, $o;
-      $ind2obj{$io->name} = $io;
-    }
-  }
-  $this->{Ind2Use} = \%ind2use;
-  $this->{Ind2Obj} = \%ind2obj;
-  $this->{IndNamesSorted} = [ sort keys %ind2use ];
-  my $i=0; my %ind2index = map +($_=>$i++), @{$this->{IndNamesSorted}};
-  $this->{Ind2Index} = \%ind2index;
-  $ind2obj{$_}->set_index($ind2index{$_}) for sort keys %ind2index;
-  @$this{qw(OtherNames OtherObjs)} = $this->_otherPars_nft($otherpars||'');
+  @$this{qw(OtherNames OtherObjs OtherAnyOut OtherFlags)} = $this->_otherPars_nft($otherpars||'');
+  my $i=0; $dimsobj->ind_obj($_)->set_index($i++) for sort $dimsobj->ind_names;
   $this;
 }
 
 sub _otherPars_nft {
     my ($sig,$otherpars) = @_;
     my $dimobjs = $sig && $sig->dims_obj;
-    my(@names,%types,$type);
-    # support 'int ndim => n;' syntax
+    my (@names,%types,$type,$any_out,%allflags);
     for (nospacesplit(';',$otherpars)) {
-	if (/^\s*([^=]+)\s*=>\s*(\S+)\s*$/) {
+	my (%flags);
+	if (s/^\s*$PDL::PP::PdlParObj::sqbr_re\s*//) {
+	  %flags = my %lflags = map +($_=>1), split /\s*,\s*/, my $opts = $1;
+	  confess "Can't have both [io] and [o]" if $lflags{o} && $lflags{io};
+	  my $this_out = delete($lflags{o}) || delete($lflags{io});
+	  confess "Invalid options '$opts' in '$_'" if keys %lflags;
+	  $any_out ||= $this_out;
+	}
+	if (/^\s*([^=]+?)\s*=>\s*(\S+)\s*$/) {
+	    # support 'int ndim => n;' syntax
 	    my ($ctype,$dim) = ($1,$2);
-	    $ctype =~ s/\s+$//; # get rid of trailing ws
 	    print "OtherPars: setting dim '$dim' from '$ctype'\n" if $::PP_VERBOSE;
 	    $type = PDL::PP::CType->new($ctype);
-	    croak "can't set unknown dimension '$dim' from '$otherpars'"
-		unless defined($dimobjs->{$dim});
-	    $dimobjs->{$dim}->set_from($type);
-	} elsif(/^\s*\(\s*void\s*\)/) {
-	    # suppressing unused param warning - skip
-	    next;
+	    $dimobjs->get_indobj_make($dim)->set_from($type);
 	} else {
 	    $type = PDL::PP::CType->new($_);
 	}
 	my $name = $type->protoname;
-	croak "Invalid OtherPars name: $name"
+	confess "Invalid OtherPars name: $name"
 	  if $PDL::PP::PdlParObj::INVALID_PAR{$name};
 	push @names,$name;
 	$types{$name} = $type;
+	$types{"${name}_count"} = PDL::PP::CType->new("PDL_Indx ${name}_count") if $type->is_array;
+	$allflags{$name} = \%flags;
     }
-    return (\@names,\%types);
+    (\@names,\%types,$any_out,\%allflags);
 }
 
-*with = \&new;
-
 =head1 AUTHOR
 
 Copyright (C) Tuomas J. Lukka 1997 (lukka@husc.harvard.edu) and by Christian
@@ -109,60 +99,71 @@ sub names_out_nca { my $o=$_[0]->objs; grep $o->{$_}{FlagOut} && !$o->{$_}{FlagC
 sub names_tmp { my $o=$_[0]->objs; grep $o->{$_}{FlagTemp}, @{$_[0]{Names}} }
 
 sub dims_obj { $_[0]->{DimsObj} }
-sub dims_count { scalar keys %{$_[0]{DimsObj}} }
-sub dims_values { values %{$_[0]{DimsObj}} }
-
-sub ind_used { $_[0]{Ind2Use}{$_[1]} }
-sub ind_obj { $_[0]{Ind2Obj}{$_[1]} }
-sub ind_names_sorted { @{$_[0]{IndNamesSorted}} }
-sub ind_index { $_[0]{Ind2Index}{$_[1]} }
+sub dims_init {
+  my ($self) = @_;
+  join "\n",
+    (sort map $_->get_initdim, $self->{DimsObj}->ind_fromcomp),
+    (sort map $_->get_initdim, $self->{DimsObj}->ind_notfromcomp);
+}
 
 sub othernames {
-  my ($self, $for_xs) = @_;
-  return $self->{OtherNames} if $for_xs;
-  my $objs = $self->otherobjs($for_xs);
-  my @raw_names = @{$self->{OtherNames}};
-  [ map $objs->{$_}->is_array ? ($_, "${_}_count") : $_, @raw_names ];
+  my ($self, $omit_count, $with_xs, $except) = @_;
+  $except ||= {};
+  return $self->{OtherNames} if $omit_count && $omit_count > 0 && !keys %$except && $with_xs;
+  return [] if $omit_count && $omit_count < 0;
+  my $objs = $self->otherobjs;
+  my @raw_names = grep !$except->{$_}, @{$self->{OtherNames}};
+  @raw_names = map $objs->{$_}->is_array ? ($_, "${_}_count") : $_, @raw_names if !$omit_count;
+  @raw_names = grep !$objs->{$_}{WasDollar}, @raw_names if !$with_xs;
+  \@raw_names;
 }
-sub otherobjs {
-  my ($self, $for_xs) = @_;
-  return $self->{OtherObjs} if $for_xs;
-  my $objs = $self->{OtherObjs};
-  my @raw_names = @{$self->{OtherNames}};
-  +{ map $objs->{$_}->is_array
-      ? ($_=>$objs->{$_}, "${_}_count"=>PDL::PP::CType->new("PDL_Indx ${_}_count"))
-      : ($_=>$objs->{$_}),
-      @raw_names };
+sub otherobjs { $_[0]{OtherObjs} }
+sub other_any_out { $_[0]{OtherAnyOut} }
+sub other_is_flag {
+  my $flag = $_[2];
+  my $has_count = (my $without_count = $_[1]) =~ s/_count$//;
+  return $_[0]{OtherFlags}{$_[1]} && $_[0]{OtherFlags}{$_[1]}{$flag} if !$has_count;
+  $_[0]{OtherFlags}{$without_count} && $_[0]{OtherFlags}{$without_count}{$flag};
 }
+sub other_is_output { &other_is_out || &other_is_io }
+sub other_is_out { $_[0]->other_is_flag($_[1], 'o') }
+sub other_out { grep $_[0]->other_is_out($_), @{$_[0]{OtherNames}} }
+sub other_is_io { $_[0]->other_is_flag($_[1], 'io') }
+sub other_io { grep $_[0]->other_is_io($_), @{$_[0]{OtherNames}} }
 
-sub allnames { [(grep !$_[0]{Objects}{$_}{FlagTemp}, @{$_[0]{Names}}), @{$_[0]->othernames($_[1])}] }
+sub allnames { my ($self, $omit_count, $with_xs, $except) = @_; [
+  ($omit_count && $omit_count < 0) ? (grep $self->{Objects}{$_}{FlagCreateAlways}, @{$self->{Names}}) :
+  (grep +(!$except || !$except->{$_}) && !$self->{Objects}{$_}{FlagTemp}, @{$self->{Names}}),
+  @{$self->othernames(@_[1..3])},
+] }
 sub allobjs {
   my $pdltype = PDL::PP::CType->new("pdl *__foo__");
-  +{ ( map +($_,$pdltype), @{$_[0]{Names}} ), %{$_[0]->otherobjs($_[1])} };
+  +{ ( map +($_,$pdltype), @{$_[0]{Names}} ), %{$_[0]->otherobjs} };
 }
 sub alldecls {
-  my ($self, $long, $for_xs) = @_;
-  return @{$self->allnames($for_xs)} if !$long;
-  my $objs = $self->allobjs($for_xs);
-  map $objs->{$_}->get_decl($_, {VarArrays2Ptrs=>1}), @{$self->allnames($for_xs)};
+  my ($self, $omit_count, $indirect, $with_xs, $except) = @_;
+  my $objs = $self->allobjs;
+  my @names = @{$self->allnames($omit_count, $with_xs, $except)};
+  $indirect = $indirect ? { map +($_=>$self->other_is_output($_)), @names } : {};
+  map $objs->{$_}->get_decl($_, {VarArrays2Ptrs=>1,AddIndirect=>$indirect->{$_}}), @names;
 }
 sub getcomp {
   my ($self) = @_;
-  my $objs = $self->otherobjs(0);
-  join '', map "$_;", grep $_, map $objs->{$_}->get_decl($_, {VarArrays2Ptrs=>1}), @{$self->othernames(0)};
+  my $objs = $self->otherobjs;
+  my @names = @{$self->othernames(0)};
+  my $indirect = { map +($_=>$self->other_is_output($_)), @names };
+  join "\n", map "  $_;", grep $_, map $objs->{$_}->get_decl($_, {VarArrays2Ptrs=>1,AddIndirect=>$indirect->{$_}}), @names;
 }
 sub getfree {
   my ($self,$symbol) = @_;
-  my $objs = $self->otherobjs(0);
+  my $objs = $self->otherobjs;
   join '', map $objs->{$_}->get_free("\$$symbol($_)",
     { VarArrays2Ptrs => 1 }), @{$self->othernames(0)};
 }
 sub getcopy {
-  my ($self) = @_;
-  my $objs = $self->otherobjs(0);
-  PDL::PP::pp_line_numbers(__LINE__,
-    join '', map $objs->{$_}->get_copy($_,"\$COMP($_)"), @{$self->othernames(0)}
-  );
+  my ($self, $to_pat) = @_;
+  my $objs = $self->otherobjs;
+  PDL::PP::indent(2, join '', map $objs->{$_}->get_copy($_,sprintf $to_pat,$_)."\n", @{$self->othernames(0)});
 }
 
 sub realdims {
@@ -172,7 +173,7 @@ sub realdims {
 
 sub creating {
   my $this = shift;
-  croak "you must perform a checkdims before calling creating"
+  confess "you must perform a checkdims before calling creating"
     unless defined $this->{Create};
   return $this->{Create};
 }
@@ -183,7 +184,7 @@ sub checkdims {
   $this->{Dims} = PDL::PP::PdlDimsObj->new;
   $this->{Objects}{$_}->add_inds($this->{Dims}) for @{$this->{Names}};
   my $n = @{$this->{Names}};
-  croak "not enough pdls to match signature" unless $#_ >= $n-1;
+  confess "not enough pdls to match signature" unless $#_ >= $n-1;
   my @pdls = @_[0..$n-1];
   if ($PDL::debug) { print "args: ".
 		     join(' ,',map { "[".join(',',$_->dims)."]," } @pdls)
