@@ -210,6 +210,9 @@ char *cpoly(double opr[], double opi[], int degree,
      the zerofinder will work provided the overflowed quantity is
      replaced by a large number. */
 
+  if (degree < 1)
+    return "algorithm only works for degree >= 1.";
+
   if (opr[0] == 0.0 && opi[0] == 0.0)
     return "algorithm fails if the leading coefficient is zero.";
 
@@ -246,7 +249,7 @@ char *cpoly(double opr[], double opi[], int degree,
   /* Make a copy of the coefficients */
   for (i=0;i<nn;i++) {
     pc[i] = opr[i] + I*opi[i];
-    shc[i] = cmod(pc[i]) + I*cimag(shc[i]);
+    shc[i] = cmod(pc[i]);
   }
 
   /* Scale the polynomial */
